"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PARTS = exports.DAYS = exports.analyzePoints = exports.fetchLeaderboard = void 0;
const https_1 = require("https");
function fetchLeaderboard(session, year, leaderboardID) {
    const url = `https://adventofcode.com/${year}/leaderboard/private/view/${leaderboardID}.json`;
    return new Promise(resolve => (0, https_1.request)(url, {
        headers: {
            cookie: `session=${session}`
        }
    }).on('response', (r) => {
        let data = '';
        r.on('data', (chunk) => {
            data += chunk;
        });
        r.on('end', () => {
            resolve(JSON.parse(data));
        });
    }).end());
}
exports.fetchLeaderboard = fetchLeaderboard;
function analyzePoints(l) {
    const events = [];
    for (const member of Object.values(l.members)) {
        for (const day of exports.DAYS) {
            const dayResults = member.completion_day_level[day];
            if (!dayResults)
                continue;
            for (const part of exports.PARTS) {
                const partResults = dayResults[part];
                if (!partResults)
                    continue;
                events.push({ member, day, part, ts: partResults.get_star_ts, points: 0 });
            }
        }
    }
    events.sort((x, y) => x.ts - y.ts);
    const pointsAvailable = new Map();
    const numMembers = Object.values(l.members).length;
    for (const event of events) {
        if (!pointsAvailable.has(event.day)) {
            pointsAvailable.set(event.day, new Map().set('1', numMembers).set('2', numMembers));
        }
        const earned = pointsAvailable.get(event.day).get(event.part);
        pointsAvailable.get(event.day).set(event.part, earned - 1);
        event.points = earned;
    }
    const available = [];
    for (const [day, partsMap] of pointsAvailable.entries()) {
        for (const [part, points] of partsMap.entries()) {
            available.push({ day, part, points });
        }
    }
    return [events, available];
}
exports.analyzePoints = analyzePoints;
exports.DAYS = ['1', '2', '3', '4', '5', '6', '7', '8', '9', '10', '11', '12', '13', '14', '15', '16', '17', '18', '19', '20', '21', '22', '23', '24', '25'];
exports.PARTS = ['1', '2'];
//# sourceMappingURL=leaderboard.js.map